package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.MaterialDao;
import br.com.centralit.api.model.Material;
import br.com.centralit.api.model.MaterialCaracteristica;
import br.com.centralit.api.model.MaterialConsumo;
import br.com.centralit.api.model.MaterialConsumoTipoUnidadeMedidaEntrada;
import br.com.centralit.api.model.MaterialImagem;
import br.com.centralit.api.model.MaterialPermanente;
import br.com.centralit.api.model.MaterialPermanenteConsumo;
import br.com.centralit.api.service.ClassificacaoMaterialService;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.MaterialService;
import br.com.centralit.api.service.ModuloService;
import br.com.centralit.api.service.UnidadeMedidaService;
import br.com.centralit.api.util.ClientRest;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Modulo;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>MaterialServiceImpl
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * @since 15/12/2014 - 08:55:28
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Service("materialService")
public class MaterialServiceImpl extends GenericServiceImpl<Material, Long> implements MaterialService {

	private static final String MSG_EXCLUSAO_NEGADA_MATERIAL_PERMANENTE = "MSG.EXCLUSAO_NEGADA_MATERIAL";
	private static final String MSG_EXCLUSAO_NEGADA_MATERIAL_CONSUMO = "MSG.EXCLUSAO_NEGADA_MATERIAL_CONSUMO";

	/** Atributo materialDao. */
	private MaterialDao materialDao;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	/** Atributo federalSupplyService. */
	@Autowired
	private ClassificacaoMaterialService federalSupplyService;

	/** Atributo moduloService. */
	@Autowired
	private ModuloService moduloService;

	/** Atributo request. */
	@Autowired
	private HttpServletRequest request;

	/** Atributo unidadeMedidaService. */
	@Autowired
	private UnidadeMedidaService unidadeMedidaService;
	
	/** Atributo configuracaoParametroSistemaService. */
	@Autowired
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param materialDao
	 */
	@Autowired
	public MaterialServiceImpl( MaterialDao materialDao, @Qualifier("materialValidator") Validator validator ) {

		this.dao = materialDao;

		this.materialDao = materialDao;

		this.validator = validator;

	}

	/**
	 * Responsável por salvar a Entidade <code>Material</code>
	 * 
	 * @param materialDao
	 */
	@Override
	public Material save(Material entity) {

		this.validarEntidade(entity, this.validator);

		this.montarDadosMaterial(entity);

		this.materialDao.save(entity);

		this.configurarCodigo(entity);
		
		return entity;
	}

	/**
	 * Responsável por alterar a Entidade <code>Material</code>
	 * 
	 * @param materialDao
	 */
	@Override
	public Material merge(Material entity) {

		this.validarEntidade(entity, this.validator);

		this.montarDadosMaterial(entity);
		
		if(!UtilObjeto.isReferencia(entity.getCodigo())){
			this.configurarCodigo(entity);
		}

		return super.merge(entity);
	}
	
	/**
	 *
	 * Método responsável gerar codigo sequencial para o material
	 *
	 * @author rogerio.cassimiro, juliana.barbosa
	 *
	 * @param material
	 */	
	private void configurarCodigo(Material entity) {

		Material ultimoRegistro = (Material) this.materialDao.buscarUltimoRegistroComOrdenadoParametrizada("codigo");

		Long sequencial = null;
		if (UtilObjeto.isReferencia(ultimoRegistro)) {
			sequencial = ultimoRegistro.getCodigo() == null ? null : Long.parseLong(ultimoRegistro.getCodigo());
		}
		String codigo = this.configuracaoParametroSistemaService.gerarNumeroIdentificacaoSeguencial(sequencial);

		entity.setCodigo(codigo);
	}	

	/**
	 * Método responsável por montar os dados do material
	 * 
	 * @author rogerio.costa
	 * 
	 * @param entity
	 */
	private void montarDadosMaterial(Material entity) {

		entity.setDominioTipoMaterial(this.dominioService.findByChaveAndCodigo("tipoMaterial", entity.getDominioTipoMaterial().getCodigo()));

		// Verifica se a lista de materialCaracterista não é null
		if (!UtilColecao.isVazio(entity.getMaterialCaracteristicas())) {
			// Percorre a lista de MaterialCaracteristica para setar o material
			for (MaterialCaracteristica materialCaracteristica : entity.getMaterialCaracteristicas()) {
				if(entity.getDominioTipoMaterial().getCodigo() == Dominio.TIPO_MATERIAL_CONSUMO){
					materialCaracteristica.setGenerico(Boolean.TRUE);
				}
				materialCaracteristica.setMaterial(entity);
			}

		} else {

			entity.setMaterialCaracteristicas(new ArrayList<MaterialCaracteristica>());

		}

		if (entity instanceof MaterialPermanente) {

			this.montarDadosMaterialPermanente(entity);

		} else if (entity instanceof MaterialConsumo) {

			this.montarDadosMaterialConsumo(entity);
		}

		// Verifica se a lista de materialImagens não é null
		if (!UtilColecao.isVazio(entity.getMaterialImagens())) {
			// Percorre a list de materialImagens para setar o material
			for (MaterialImagem materialImagem : entity.getMaterialImagens()) {

				materialImagem.setMaterial(entity);

			}

		} else {

			entity.setMaterialImagens(new ArrayList<MaterialImagem>());
		}

		entity.setClassificacaoMaterial(this.federalSupplyService.find(entity.getClassificacaoMaterial().getId()));

		StringBuilder sb = new StringBuilder();

		sb.append(entity.getClassificacaoMaterial().getClassificacaoMaterialParent().getDescricao()).append(" - ").append(entity.getClassificacaoMaterial().getDescricao());

		entity.setDescricao(sb.toString());
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author rogerio.costa
	 * 
	 * @param entity
	 */
	private void montarDadosMaterialConsumo(Material entity) {

		( (MaterialConsumo) entity ).setUnidadeArmazenamento(this.unidadeMedidaService.getReference(( (MaterialConsumo) entity ).getUnidadeArmazenamento().getId()));

		Collection<MaterialConsumoTipoUnidadeMedidaEntrada> listaMaterialConsumoUnidadeMedidaEntrada = ( (MaterialConsumo) entity ).getTiposUnidadeMedidaEntrada();

		if (!UtilColecao.isVazio(listaMaterialConsumoUnidadeMedidaEntrada)) {

			for (MaterialConsumoTipoUnidadeMedidaEntrada materialConsumoTipoUnidadeMedidaEntrada : listaMaterialConsumoUnidadeMedidaEntrada) {

				materialConsumoTipoUnidadeMedidaEntrada.setMaterialConsumo((MaterialConsumo) entity);

			}

		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author rogerio.costa
	 * 
	 * @param entity
	 */
	private void montarDadosMaterialPermanente(Material entity) {

		( (MaterialPermanente) entity ).setUnidadeMedida(this.unidadeMedidaService.getReference(( (MaterialPermanente) entity ).getUnidadeMedida().getId()));

		Collection<MaterialPermanenteConsumo> listaMaterialConsumo = ( (MaterialPermanente) entity ).getMateriaisConsumo();

		// Verifica se a lista de materiaisConsumo não é null
		if (!UtilColecao.isVazio(listaMaterialConsumo)) {
			// Percorre a lista de materiaisConsumo para setar o material
			for (MaterialPermanenteConsumo materialConsumo : listaMaterialConsumo) {

				materialConsumo.setMaterialPermanente((MaterialPermanente) entity);

			}

		} else {

			( (MaterialPermanente) entity ).setMateriaisConsumo(new ArrayList<MaterialPermanenteConsumo>());
		}
	}

	/**
	 * Método responsável por listar a endidade <code>Material</code>
	 * 
	 * @author rogerio.costa
	 * 
	 * @param nome
	 * 
	 * @return Collection<Material>
	 */
	public Collection<Material> listarMaterial(String codigoOuDescricao, Long dominioCodigoTipoMaterial) {

		Usuario usuarioLogado = ( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() );

		return this.getMaterialDao().listarMaterial(codigoOuDescricao, dominioCodigoTipoMaterial, usuarioLogado.getOrganizacao().getId());

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter a Entidade<code>Material</code> através do ID da classificacaoMaterial
	 * 
	 * @author rogerio.costa
	 * 
	 * @return Material
	 */
	public Material obterPorIdClassificacaoMaterial(Long idClassificacaoMaterial) {

		return this.getMaterialDao().obterPorIdClassificacaoMaterial(idClassificacaoMaterial);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean existeMaterialVinculadoAClassificacao(Set<Long> idsClassificacoes, Usuario usuarioLogado) {

		return this.materialDao.existeMaterialVinculadoAClassificacao(idsClassificacoes, usuarioLogado);
	}

	@Override
	public boolean removeById(Long id) {

		Material material = find(id);
		
		boolean existeVinculo = false;
		
		if (material.getDominioTipoMaterial().getCodigo().equals(Dominio.TIPO_MATERIAL_PERMANENTE)) {
			
			existeVinculo = existeVinculoBemPatrimonial(id);
			lancarBusinessException(MSG_EXCLUSAO_NEGADA_MATERIAL_PERMANENTE, CodigoErro.REGRA_NEGOCIO.getValue(), existeVinculo);
		} else if (material.getDominioTipoMaterial().getCodigo().equals(Dominio.TIPO_MATERIAL_CONSUMO)){
			
			existeVinculo = existeVinculoCatalogoMaterialOuEntrada(id);
			lancarBusinessException(MSG_EXCLUSAO_NEGADA_MATERIAL_CONSUMO, CodigoErro.REGRA_NEGOCIO.getValue(), existeVinculo);
		}

		return super.removeById(id);
	}

	private boolean existeVinculoCatalogoMaterialOuEntrada(Long id) {
		Modulo moduloAlmoxarifado = moduloService.moduloEstaAtivo(ClientRest.MODULO_ALMOXARFADO);
		
		boolean contemCatalogacaoOuMovimento = false;
		if (UtilObjeto.isReferencia(moduloAlmoxarifado)) {
			ClientRest clientRestAlmoxarifado = new ClientRest(request, moduloAlmoxarifado);
			clientRestAlmoxarifado.addParametro("id", id.toString());
			clientRestAlmoxarifado.addParametro("joinClass", "material.id");

			
			boolean contemEntrada = clientRestAlmoxarifado.validarRegra("entradaAlmoxarifadoItem/existeVinculo");

			clientRestAlmoxarifado.addParametro("joinClass", "materialConsumo.id");
			boolean contemCatalogacao = clientRestAlmoxarifado.validarRegra("catalogoMaterialItem/existeVinculo");
			
			contemCatalogacaoOuMovimento = (contemEntrada || contemCatalogacao);
		}
		
		return contemCatalogacaoOuMovimento;
	}

	private boolean existeVinculoBemPatrimonial(Long id) {
		Modulo moduloPatrimonio = moduloService.moduloEstaAtivo(ClientRest.MODULO_PATRIMONIO);

		boolean contemBemPatrimonial = false;
		if (UtilObjeto.isReferencia(moduloPatrimonio)) {

			ClientRest clientRestPatrimonio = new ClientRest(request, moduloPatrimonio);
			clientRestPatrimonio.addParametro("id", id.toString());
			clientRestPatrimonio.addParametro("joinClass", "material.id");

			 contemBemPatrimonial = clientRestPatrimonio.validarRegra("bemPatrimonial/existeVinculo");
		}
		
		return contemBemPatrimonial;
	}

	/**
	 * Retorna o valor do atributo <code>materialDao</code>
	 * 
	 * @return <code>MaterialDao</code>
	 */
	public MaterialDao getMaterialDao() {

		return materialDao;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por
	 * 
	 * @author iago.almeida
	 * 
	 * @param idContaContabil
	 * @return
	 */
	@Override
	public boolean existeMaterialVinculadoAContaContabil(Long idContaContabil, Usuario usuarioLogado) {

		return this.materialDao.existeMaterialVinculadoAContaContabil(idContaContabil, usuarioLogado);
	}

	@Override
	public Collection<Long> getIdsContaContabilEmUso(Long idOrganizacao) {

		return this.materialDao.getIdsContasContabeisEmMateriais(idOrganizacao);
	}

	/**
	 * Método responsável por consultar um materiais pelo código ou pelo nome.
	 * 
	 * @author luis.camargo
	 * 
	 * @param codigoOuDescricao
	 * @param idOrganizacao
	 * 
	 * @return
	 */
	public Collection<Material> findMateriaisByOrganizacao(String codigoOuDescricao, Long idOrganizacao) {
		
		return this.materialDao.findMateriaisByOrganizacao(codigoOuDescricao, idOrganizacao);
	}

}
